import {IllegalAction} from "./illegalAction.js"
import {roomManager} from "../roomManager.js"
import {io} from "./io.js"
import {Player} from "../player.js"
import {Game} from "../game.js"

io.on('connection', function(client:any){
  connected(client)
});


function connected(client:any) {
    client.sync = function() {
      const tempWorld = client.game.world.obfuscate();
      io.to(client.game.id).emit('sync',{world: tempWorld})
    }

    client.sendMeta = function() {
      const roomID        = client.game.id
      const roomPlayers   = client.game.players; // Dont send client id's to everyone, TODO
      const metadata      = {players: roomPlayers, roomID: roomID}
      io.to(client.game.id).emit('metadata', metadata)
      const id = client.game.id
      client.emit('message', id)
    }
    client.game = null;
    client.player = null;

    client.on('disconnect', function(){
      if (!roomManager.getAllPlayerIDs().includes(client.id)) return new IllegalAction(client.id, 22)
      client.game.removePlayerByID(client.id)
      client.sendMeta();
      client.game = null
      client.player = null
    })

    client.on('joinGame', function(data:any){
        const info = roomManager.playerJoinGame(data.room, data.name, data.color, client.id)
        client.join(info[0].id)
        client.game = info[0];
        client.player = info[1]
        client.sync()
        client.emit('inGame', true)
        client.sendMeta()
    })
    type CreateGameOptions = {width: number, height: number, mines: number}
    type CreateGameData = {options: CreateGameOptions, name: string, color: number}

    client.on('createGame', function(data:any){
        if (
          typeof data.name === "string" &&
          typeof data.color === "number" &&
          typeof data.options.width === "number" &&
          typeof data.options.height === "number" &&
          typeof data.options.mines === "number"
        ) {
          console.log("good 2")
        } else {return false}
        if (
          data.name.length < 9 &&
          data.color > 0 && data.color < 9 &&
          data.options.width > 8 && data.options.width < 500 &&
          data.options.height > 8 && data.options.height < 500 &&
          data.options.mines > 6 && data.options.mines < (data.options.height * data.options.width)
        ) {
          console.log("good 3")
        } else {return false}
        const createGameData: CreateGameData = data as CreateGameData;
        console.log(createGameData)
        const info = roomManager.playerCreateGame(createGameData.options, createGameData.name, createGameData.color, client.id)
        client.join(info[0].id)
        client.game = info[0];
        client.player = info[1]
        client.emit('inGame', true)
        client.sync()
        client.sendMeta()
    })

    client.on('leaveGame', function(data:any){
      client.game.removePlayerByID(client.id)
    })

    client.on('clickCanvas', function(data:any){
      if (!roomManager.getAllPlayerIDs().includes(client.id)) return new IllegalAction(client.id, 1)
      client.game.world.click(data.tilePosition[0],data.tilePosition[1], data.mode, client.player.color)
      client.sync()
    })
}
